package cn.com.qmth.scancloud.tools.service.impl;

import cn.com.qmth.scancloud.tools.config.SysProperty;
import cn.com.qmth.scancloud.tools.enums.TaskType;
import cn.com.qmth.scancloud.tools.model.Course;
import cn.com.qmth.scancloud.tools.service.AbstractTask;
import cn.com.qmth.scancloud.tools.service.CommonService;
import cn.com.qmth.scancloud.tools.utils.FileHelper;
import cn.com.qmth.scancloud.tools.utils.HttpHelper;
import cn.com.qmth.scancloud.tools.utils.JsonHelper;
import cn.com.qmth.scancloud.tools.utils.StatusException;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.collections4.MapUtils;
import org.apache.commons.io.LineIterator;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * 课程导入
 */
public class CourseImportTask extends AbstractTask {

    private static final Logger log = LoggerFactory.getLogger(CourseImportTask.class);

    @Override
    protected String getTaskName() {
        return TaskType.COURSE_IMPORT.getTitle();
    }

    @Override
    protected void execute() {
        Long examId = SysProperty.EXAM_ID;
        if (examId == null) {
            throw new StatusException("【scan.tool.examId】未配置！");
        }

        if (CommonService.findExam(examId) == null) {
            throw new StatusException("当前考试不存在！examId = " + examId);
        }

        if (!SysProperty.DATA_FILE.endsWith(".txt")) {
            throw new StatusException("导入模板目前仅支持后缀名为“.txt”的文件！");
        }

        // 自定义模板 列分隔符
        String templateSeparator = StringUtils.isNotEmpty(SysProperty.TEMPLATE_SEPARATOR) ? SysProperty.TEMPLATE_SEPARATOR : "|";
        // 自定义模板 列与索引位置
        Map<String, Integer> templateColumns = JsonHelper.toMap(SysProperty.TEMPLATE_COLUMNS, Integer.class);

        int total = 0;
        List<Course> list = new ArrayList<>();
        File file = new File(SysProperty.DATA_FILE);
        try (LineIterator lines = FileHelper.readLines(file);) {
            while (lines.hasNext()) {
                String line = lines.nextLine();

                Course data = this.parseValues(total, line, templateColumns, templateSeparator);
                if (data != null) {
                    data.setExamId(examId);
                    list.add(data);
                }

                total++;
            }
        } catch (IOException e) {
            log.error(e.getMessage(), e);
        }

        log.info("导入文件全部行数共 {} 行，实际有效数据共 {} 条！", total, list.size());
        if (CollectionUtils.isEmpty(list)) {
            return;
        }

        String url = SysProperty.SCAN_SERVER_URL + "/api/tool/import/course";
        String json = JsonHelper.toJson(list);
        String result = HttpHelper.post(url, json);
        log.info("已处理数：{} 进度：100% {}", list.size(), result);
    }

    private Course parseValues(int index, String line, Map<String, Integer> templateColumns, String templateSeparator) {
        if (StringUtils.isBlank(line)) {
            return null;
        }

        line = StringUtils.deleteWhitespace(line);
        String[] values = StringUtils.splitByWholeSeparatorPreserveAllTokens(line, templateSeparator);

        try {
            String subjectCode = null;
            String subjectName = null;

            if (MapUtils.isEmpty(templateColumns)) {
                // 默认模板：科目代码|科目名称
                subjectCode = values[0];
                subjectName = values[1];
            } else {
                // 自定义模板
                if (templateColumns.get("科目代码") != null) {
                    subjectCode = values[templateColumns.get("科目代码")];
                }
                if (templateColumns.get("科目名称") != null) {
                    subjectName = values[templateColumns.get("科目名称")];
                }
            }

            if (StringUtils.isEmpty(subjectCode)) {
                throw new StatusException(String.format("【第%s行】“科目代码”字段不能为空！ %s", index + 1, line));
            }
            if (StringUtils.isEmpty(subjectName)) {
                throw new StatusException(String.format("【第%s行】“科目名称”字段不能为空！ %s", index + 1, line));
            }

            Course data = new Course();
            data.setSubjectCode(subjectCode);
            data.setSubjectName(subjectName);
            return data;
        } catch (ArrayIndexOutOfBoundsException e) {
            log.error("【第{}行】内容格式错误！ {} {}", index + 1, line, e.toString());
            throw new StatusException("数据内容有误！");
        }
    }

}
