package cn.com.qmth.scancloud.tools.service.impl;

import cn.com.qmth.scancloud.tools.config.SysProperty;
import cn.com.qmth.scancloud.tools.enums.ExamMode;
import cn.com.qmth.scancloud.tools.enums.ExamType;
import cn.com.qmth.scancloud.tools.enums.TaskType;
import cn.com.qmth.scancloud.tools.model.Exam;
import cn.com.qmth.scancloud.tools.service.AbstractTask;
import cn.com.qmth.scancloud.tools.utils.FileHelper;
import cn.com.qmth.scancloud.tools.utils.HttpHelper;
import cn.com.qmth.scancloud.tools.utils.JsonHelper;
import cn.com.qmth.scancloud.tools.utils.StatusException;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.collections4.MapUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * 考试导入
 */
public class ExamImportTask extends AbstractTask {

    private static final Logger log = LoggerFactory.getLogger(ExamImportTask.class);

    @Override
    protected String getTaskName() {
        return TaskType.EXAM_IMPORT.getTitle();
    }

    @Override
    protected void execute() {
        Long schoolId = SysProperty.SCHOOL_ID;
        if (schoolId == null) {
            throw new StatusException("【scan.tool.schoolId】未配置！");
        }
        String schoolName = SysProperty.SCHOOL_NAME;
        if (StringUtils.isBlank(schoolName)) {
            throw new StatusException("【scan.tool.schoolName】未配置！");
        }
        if (!SysProperty.DATA_FILE.endsWith(".txt")) {
            throw new StatusException("导入模板目前仅支持后缀名为“.txt”的文件！");
        }

        // 自定义模板 列分隔符
        String templateSeparator = StringUtils.isNotEmpty(SysProperty.TEMPLATE_SEPARATOR)
                ? SysProperty.TEMPLATE_SEPARATOR
                : "|";
        // 自定义模板 列与索引位置
        Map<String, Integer> templateColumns = JsonHelper.toMap(SysProperty.TEMPLATE_COLUMNS, Integer.class);

        File file = new File(SysProperty.DATA_FILE);
        List<String> lines = FileHelper.readAllLines(file);
        if (CollectionUtils.isEmpty(lines)) {
            log.warn("导入文件内无数据！");
            return;
        }

        List<Exam> list = new ArrayList<>();
        for (int n = 0; n < lines.size(); n++) {
            Exam data = this.parseValues(n, lines.get(n), templateColumns, templateSeparator);
            if (data != null) {
                data.setSchoolId(schoolId);
                data.setSchoolName(schoolName);
                list.add(data);
            }
        }

        log.info("导入文件全部行数共 {} 行，实际有效数据共 {} 条！", lines.size(), list.size());
        if (CollectionUtils.isEmpty(list)) {
            return;
        }

        String url = SysProperty.SCAN_SERVER_URL + "/api/tool/import/exam";
        for (int n = 0; n < list.size(); n++) {
            String json = JsonHelper.toJson(list.get(n));
            String result = HttpHelper.post(url, json);

            float rate = (n + 1) * 100f / list.size();
            log.info("已处理数：{} 进度：{}% {}", n + 1, rate, result);
        }
    }

    private Exam parseValues(int index, String line, Map<String, Integer> templateColumns, String templateSeparator) {
        if (StringUtils.isBlank(line)) {
            return null;
        }

        line = StringUtils.deleteWhitespace(line);
        String[] values = StringUtils.splitByWholeSeparatorPreserveAllTokens(line, templateSeparator);

        try {
            Long examId = null;
            String examName = null;
            ExamType examType = null;

            if (MapUtils.isEmpty(templateColumns)) {
                // 默认模板：考试ID|考试名称|考试类型
                examId = this.parseLong(values[0]);
                examName = values[1];
                examType = ExamType.findByName(values[2]);
            } else {
                // 自定义模板
                if (templateColumns.get("考试ID") != null) {
                    examId = this.parseLong(values[templateColumns.get("考试ID")]);
                }
                if (templateColumns.get("考试名称") != null) {
                    examName = values[templateColumns.get("考试名称")];
                }
                if (templateColumns.get("考试类型") != null) {
                    examType = ExamType.findByName(values[templateColumns.get("考试类型")]);
                }
            }

            if (examId == null) {
                throw new StatusException(String.format("【第%s行】“考试ID”字段值有误！ %s", index + 1, line));
            }
            if (StringUtils.isEmpty(examName)) {
                throw new StatusException(String.format("【第%s行】“考试名称”字段不能为空！ %s", index + 1, line));
            }
            if (examType == null) {
                throw new StatusException(String.format("【第%s行】“考试类型”字段值有误！ %s", index + 1, line));
            }

            Exam data = new Exam();
            data.setId(examId);
            data.setName(examName);

            data.setMode(ExamMode.CET);
            data.setScanByPackage(true);
            data.setEnableSyncVerify(false);
            data.setAllowUnexistPaper(false);
            data.setAnswerPaperNumberFigure(0);
            data.setAnswerFrontCardType(1);
            data.setEnableSinglePageAnswer(false);

            if (StringUtils.isNotBlank(SysProperty.PAPER_TYPE_BARCODE)) {
                data.setPaperTypeBarcodeContent(Arrays.asList(SysProperty.PAPER_TYPE_BARCODE.split(",")));
            }

            // CET时“卷型条码规则”不能为空，且必须为6位数字！
            if (CollectionUtils.isEmpty(data.getPaperTypeBarcodeContent())) {
                throw new StatusException("【exam.cet.params】配置“卷型条码规则”为空！");
            }
            for (String barcode : data.getPaperTypeBarcodeContent()) {
                if (StringUtils.length(barcode) != 6 || parseLong(barcode) == null) {
                    throw new StatusException("【exam.cet.params】配置“卷型条码规则”值必须为6位数字！");
                }
            }

            if (data.getAbsentBarcodeContent() == null) {
                data.setAbsentBarcodeContent("");
            }

            if (data.getPaperTypeBarcodeContent() == null) {
                data.setPaperTypeBarcodeContent(new ArrayList<>());
            }

            return data;
        } catch (ArrayIndexOutOfBoundsException e) {
            log.error("【第{}行】内容格式错误！ {} {}", index + 1, line, e.toString());
            throw new StatusException("数据内容有误！");
        }
    }

    private Long parseLong(String str) {
        try {
            return Long.parseLong(str);
        } catch (Exception e) {
            return null;
        }
    }

}
