package cn.com.qmth.scancloud.tools.service.impl;

import cn.com.qmth.scancloud.tools.config.SysProperty;
import cn.com.qmth.scancloud.tools.enums.RoleType;
import cn.com.qmth.scancloud.tools.enums.TaskType;
import cn.com.qmth.scancloud.tools.model.User;
import cn.com.qmth.scancloud.tools.service.AbstractTask;
import cn.com.qmth.scancloud.tools.utils.FileHelper;
import cn.com.qmth.scancloud.tools.utils.HttpHelper;
import cn.com.qmth.scancloud.tools.utils.JsonHelper;
import cn.com.qmth.scancloud.tools.utils.StatusException;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.collections4.MapUtils;
import org.apache.commons.io.LineIterator;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * 管理员用户导入
 */
public class UserImportTask extends AbstractTask {

    private static final Logger log = LoggerFactory.getLogger(UserImportTask.class);

    @Override
    protected String getTaskName() {
        return TaskType.USER_IMPORT.getTitle();
    }

    @Override
    protected void execute() {
        Long schoolId = SysProperty.SCHOOL_ID;
        if (schoolId == null) {
            throw new StatusException("【scan.tool.schoolId】未配置！");
        }
        if (!SysProperty.DATA_FILE.endsWith(".txt")) {
            throw new StatusException("导入模板目前仅支持后缀名为“.txt”的文件！");
        }

        // 自定义模板 列分隔符
        String templateSeparator = StringUtils.isNotEmpty(SysProperty.TEMPLATE_SEPARATOR) ? SysProperty.TEMPLATE_SEPARATOR : "|";
        // 自定义模板 列与索引位置
        Map<String, Integer> templateColumns = JsonHelper.toMap(SysProperty.TEMPLATE_COLUMNS, Integer.class);

        int total = 0;
        List<User> list = new ArrayList<>();
        File file = new File(SysProperty.DATA_FILE);
        try (LineIterator lines = FileHelper.readLines(file);) {
            while (lines.hasNext()) {
                String line = lines.nextLine();

                User data = this.parseValues(total, line, templateColumns, templateSeparator);
                if (data != null) {
                    data.setSchoolId(schoolId);
                    list.add(data);
                }

                total++;
            }
        } catch (IOException e) {
            log.error(e.getMessage(), e);
        }

        log.info("导入文件全部行数共 {} 行，实际有效数据共 {} 条！", total, list.size());
        if (CollectionUtils.isEmpty(list)) {
            return;
        }

        String url = SysProperty.SCAN_SERVER_URL + "/api/tool/import/user";
        String json = JsonHelper.toJson(list);
        String result = HttpHelper.post(url, json);
        log.info("已处理数：{} 进度：100% {}", list.size(), result);
    }

    private User parseValues(int index, String line, Map<String, Integer> templateColumns, String templateSeparator) {
        if (StringUtils.isBlank(line)) {
            return null;
        }

        line = StringUtils.deleteWhitespace(line);
        String[] values = StringUtils.splitByWholeSeparatorPreserveAllTokens(line, templateSeparator);

        try {
            String name = null;
            String loginName = null;
            String password = null;

            if (MapUtils.isEmpty(templateColumns)) {
                // 默认模板：姓名|账号|密码
                name = values[0];
                loginName = values[1];
                password = values[2];
            } else {
                // 自定义模板
                if (templateColumns.get("姓名") != null) {
                    name = values[templateColumns.get("姓名")];
                }
                if (templateColumns.get("账号") != null) {
                    loginName = values[templateColumns.get("账号")];
                }
                if (templateColumns.get("密码") != null) {
                    password = values[templateColumns.get("密码")];
                }
            }

            if (StringUtils.isEmpty(loginName)) {
                throw new StatusException(String.format("【第%s行】“账号”字段不能为空！ %s", index + 1, line));
            }
            if (StringUtils.isEmpty(password)) {
                throw new StatusException(String.format("【第%s行】“密码”字段不能为空！ %s", index + 1, line));
            }
            if (StringUtils.isEmpty(name)) {
                name = loginName;
            }

            User data = new User();
            data.setName(name);
            data.setLoginName(loginName);
            data.setPassword(password);
            // 默认“学校管理员”
            data.setRole(RoleType.SCHOOL_ADMIN);
            return data;
        } catch (ArrayIndexOutOfBoundsException e) {
            log.error("【第{}行】内容格式错误！ {} {}", index + 1, line, e.toString());
            throw new StatusException("数据内容有误！");
        }
    }

}
